/** @file
 * @brief GNUDIP Service class header. This class represents the GNUDIP service.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#ifndef ServiceGnudip_H
#define ServiceGnudip_H

#include "service.h"
#include "logger.h"
#include "httphelper.h"

#include <boost/shared_ptr.hpp>
#include <map>
#include <string>


class ServiceGnudip : public Service
{

private:

    friend class boost::serialization::access;
    template<class Archive>
    void serialize(Archive & ar, const unsigned int version)
    {
        ar & boost::serialization::base_object<Service>(*this);
    }

    std::string GnudipServer;
    std::string BaseUrl;

    HTTPHelper::Ptr HTTPHelp;

    std::string assemble_base_url(const std::string& fqhn, const std::string& gnudip_server) const;
    std::map<std::string,std::string> parse_initial_request(const std::string& curl_data) const;
    std::string assemble_update_url(const std::string& salt, const std::string& time, const std::string& sign, const std::string& secret, const std::string& ip) const;

public:

    typedef boost::shared_ptr<ServiceGnudip> Ptr;

    ServiceGnudip();

    ServiceGnudip(const std::string& _protocol, const std::string& _gnudip_server ,const std::string& _hostname, const std::string& _login, const std::string& _password, const Logger::Ptr& _logger, const int _update_interval, const int _max_updates_within_interval, const int dns_cache_ttl, const std::string& proxy, const int proxy_port);

    ~ServiceGnudip();

    int perform_update(const std::string& ip);

};

#endif
